# PLATFORM_HOST = 'http://127.0.0.1:3002'
PLATFORM_HOST = 'https://o5mdw8b6mh.execute-api.us-east-1.amazonaws.com/test'
API_VERSION = '/api/v1'
NET = "T"

######## TESTING #################
MOCK_PAYMENT_SERVER = "http://127.0.0.1:3000"
