VSYS = 100000000
DEFAULT_TX_FEE = int(0.1 * VSYS)
DEFAULT_FEE_SCALE = 100

DEFAULT_PAYMENT_FEE = DEFAULT_TX_FEE
DEFAULT_LEASE_FEE = DEFAULT_TX_FEE
DEFAULT_CANCEL_LEASE_FEE = DEFAULT_TX_FEE
CHECK_FEE_SCALE = True

MAX_ATTACHMENT_SIZE = 140
MAX_NONCE = 4294967295
MAX_TX_HISTORY_LIMIT = 10000

DEFAULT_CHAIN = 'mainnet'
DEFAULT_CHAIN_ID = 'M'

TESTNET_CHAIN = 'testnet'
TESTNET_CHAIN_ID = 'T'

DEFAULT_TESTNET_NODE = 'https://test.v.systems/api'
DEFAULT_TESTNET_API_KEY = ''

DEFAULT_NODE = 'http://54.147.255.148:9922'
DEFAULT_NODE_API_KEY= ''

ADDRESS_VERSION = 5
ADDRESS_CHECKSUM_LENGTH = 4
ADDRESS_HASH_LENGTH = 20
ADDRESS_LENGTH = 1 + 1 + ADDRESS_CHECKSUM_LENGTH + ADDRESS_HASH_LENGTH

PAYMENT_TX_TYPE = 2
LEASE_TX_TYPE = 3
LEASE_CANCEL_TX_TYPE = 4

THROW_EXCEPTION_ON_ERROR = True
